@@ -8,13 +8,16 @@ use File::Spec ();
 use Test::More ();
 
 use vars qw{$VERSION @ISA @EXPORT @CALL_FUNCS};
+my $parent;
 BEGIN {
-	$VERSION = '1.29';
+	$VERSION = '1.31';
 	@ISA     = 'Exporter';
 	@EXPORT  = qw/connect_ok dies @CALL_FUNCS/;
 
 	# Allow tests to load modules bundled in /inc
 	unshift @INC, 'inc';
+
+	$parent = $$;
 }
 
 # Always load the DBI module
@@ -22,6 +25,8 @@ use DBI ();
 
 # Delete temporary files
 sub clean {
+	return
+		if $$ != $parent;
 	unlink( 'foo'         );
 	unlink( 'foo-journal' );
 }
